#include <ap_cint.h>
#include <ap_utils.h>
#include "synth_settings.h"

////////////////////////////////////////////
// port
////////////////////////////////////////////

// CAN #0 side
volatile uint20	can_0_sec;
volatile uint24	can_0_secfrac;
volatile uint16	can_0_id;
volatile uint8	can_0_length;
volatile uint64	can_0_data;
volatile uint1	can_0_dout_ena;				// IN
volatile uint1	can_0_dout_req	= 0;		// OUT
volatile uint1	can_0_dout_ack;				// IN
volatile uint16	can_0_numdata;

// packet out side
volatile uint10	pct_dout		= 0;		// OUT
volatile uint16	pct_dout_numspace;			// IN
volatile uint1	pct_dout_wr		= 0;		// OUT

// dummy
volatile uint1	dummy_tmr_out	= 0;

////////////////////////////////////////////
// sub functions
////////////////////////////////////////////
uint8	pct_buf_g[256 + 16];				// +16 ... header space

void wait_tmr(uint32 tmr)	// unit clk
{
	uint32	t;
	ap_wait();
	for (t = 0; t < tmr; t++) {
		dummy_tmr_out	= 1- dummy_tmr_out;
	}
	ap_wait();
}


void send_packet(uint16 len)
{
	uint16	i;
	uint10	dout_val;

	if (len == 0)
		return;

	do {
		ap_wait();
	} while (pct_dout_numspace < len);

	pct_dout_wr	= 0;
	ap_wait();

	for (i = 0; i < len; i++) {
		dout_val	= (uint10)(pct_buf_g[i]);
		if (i == 0)
			dout_val	|= 0x200;	// start mark
		else if (i == len - 1)
			dout_val	|= 0x100;	// end mark

		pct_dout	= dout_val;
		ap_wait();
		pct_dout_wr	= 1;
		ap_wait();
		pct_dout_wr	= 0;
	}
}


uint8 bin2char(uint4 val)
{
	switch (val) {
	case 0:		return '0'; break;
	case 1:		return '1'; break;
	case 2:		return '2'; break;
	case 3:		return '3'; break;
	case 4:		return '4'; break;
	case 5:		return '5'; break;
	case 6:		return '6'; break;
	case 7:		return '7'; break;
	case 8:		return '8'; break;
	case 9:		return '9'; break;
	case 10:	return 'A'; break;
	case 11:	return 'B'; break;
	case 12:	return 'C'; break;
	case 13:	return 'D'; break;
	case 14:	return 'E'; break;
	default:	return 'F';
	}
	return 0;
}


////////////////////////////////////////////
// main
////////////////////////////////////////////
void data_formatter(void)
{
#pragma HLS TOP
#pragma HLS INTERFACE ap_none port=dummy_tmr_out	register

#pragma HLS INTERFACE ap_none port=can_0_sec
#pragma HLS INTERFACE ap_none port=can_0_secfrac
#pragma HLS INTERFACE ap_none port=can_0_id
#pragma HLS INTERFACE ap_none port=can_0_length
#pragma HLS INTERFACE ap_none port=can_0_data
#pragma HLS INTERFACE ap_none port=can_0_dout_ena
#pragma HLS INTERFACE ap_none port=can_0_dout_req	register
#pragma HLS INTERFACE ap_none port=can_0_dout_ack
#pragma HLS INTERFACE ap_none port=can_0_numdata

#pragma HLS INTERFACE ap_none port=pct_dout			register
#pragma HLS INTERFACE ap_none port=pct_dout_numspace
#pragma HLS INTERFACE ap_none port=pct_dout_wr		register

	uint16	pct_buf_ptr;		// note: not 8bit
	uint8	cnt;
	uint16	i;

	////////////////////////////////////////
	// init ports
	////////////////////////////////////////
	ap_wait();
	can_0_dout_req	= 0;
	pct_dout_wr		= 0;
	ap_wait();

	////////////////////////////////////////
	// main loop
	////////////////////////////////////////
	while (1) {
		/////////////////////////////////////
		// check CAN #0
		/////////////////////////////////////
		if (can_0_numdata >= 1 && can_0_dout_ena == 1) {
			// read fifo (output will be latched)
			do {
				can_0_dout_req	= 1;
				ap_wait();
			} while (can_0_dout_ack == 0);
			// (now fifo output is available)
			can_0_dout_req	= 0;
			ap_wait();
			do {
				ap_wait();
			} while (can_0_dout_ack == 1);

			// construct packet
			// sample: (1545298054.573393) can0 040#006400640000789E
			pct_buf_ptr = 0;
			pct_buf_g[pct_buf_ptr++]	= '(';
			pct_buf_g[pct_buf_ptr++]	= bin2char(apint_get_range(can_0_sec, 19, 16));
			pct_buf_g[pct_buf_ptr++]	= bin2char(apint_get_range(can_0_sec, 15, 12));
			pct_buf_g[pct_buf_ptr++]	= bin2char(apint_get_range(can_0_sec, 11,  8));
			pct_buf_g[pct_buf_ptr++]	= bin2char(apint_get_range(can_0_sec,  7,  4));
			pct_buf_g[pct_buf_ptr++]	= bin2char(apint_get_range(can_0_sec,  3,  0));
			pct_buf_g[pct_buf_ptr++]	= '.';
			pct_buf_g[pct_buf_ptr++]	= bin2char(apint_get_range(can_0_secfrac, 23, 20));
			pct_buf_g[pct_buf_ptr++]	= bin2char(apint_get_range(can_0_secfrac, 19, 16));
			pct_buf_g[pct_buf_ptr++]	= bin2char(apint_get_range(can_0_secfrac, 15, 12));
			pct_buf_g[pct_buf_ptr++]	= bin2char(apint_get_range(can_0_secfrac, 11,  8));
			pct_buf_g[pct_buf_ptr++]	= bin2char(apint_get_range(can_0_secfrac,  7,  4));
			pct_buf_g[pct_buf_ptr++]	= bin2char(apint_get_range(can_0_secfrac,  3,  0));
			pct_buf_g[pct_buf_ptr++]	= ')';
			pct_buf_g[pct_buf_ptr++]	= ' ';
			pct_buf_g[pct_buf_ptr++]	= 'c';
			pct_buf_g[pct_buf_ptr++]	= 'a';
			pct_buf_g[pct_buf_ptr++]	= 'n';
			pct_buf_g[pct_buf_ptr++]	= '0';
			pct_buf_g[pct_buf_ptr++]	= ' ';
			pct_buf_g[pct_buf_ptr++]	= bin2char(apint_get_range(can_0_id, 11,  8));
			pct_buf_g[pct_buf_ptr++]	= bin2char(apint_get_range(can_0_id,  7,  4));
			pct_buf_g[pct_buf_ptr++]	= bin2char(apint_get_range(can_0_id,  3,  0));
			pct_buf_g[pct_buf_ptr++]	= '#';

			if (can_0_length > 0) {
				uint64	val;
				val	= can_0_data;	// load data

				for (i = 0; i < can_0_length; i++) {
					pct_buf_g[pct_buf_ptr++]	= bin2char(apint_get_range(val, 63, 60));
					val	<<= 4;
					pct_buf_g[pct_buf_ptr++]	= bin2char(apint_get_range(val, 63, 60));
					val	<<= 4;
				}
			}

			pct_buf_g[pct_buf_ptr++]	= 0x0D;
			pct_buf_g[pct_buf_ptr++]	= 0x0A;

			// send
			send_packet(pct_buf_ptr);
		}

		ap_wait();
	}
}
