#include <ap_cint.h>
#include <ap_utils.h>

#define	CLK_FREQ_MHZ	50
#define	CLK_FREQ_KHZ	50000
#define	CLK_FREQ_HZ		50000000

////////////////////////////////////////////
// port
////////////////////////////////////////////

// UART
volatile uint1	uart_rx;						// IN: synchronized input

// GPS output
volatile uint5	gps_time_hour		= 0;
volatile uint6	gps_time_min		= 0;
volatile uint6	gps_time_sec		= 0;
//volatile uint10	gps_time_msec		= 0;
//volatile uint8	gps_latitude_deg	= 0;
//volatile uint20	gps_latitude_min	= 0;
//volatile uint1	gps_latitude_ns		= 0;
//volatile uint8	gps_longitude_deg	= 0;
//volatile uint20	gps_longitude_min	= 0;
//volatile uint1	gps_longitude_ew	= 0;
//volatile uint1	gps_alt_ant_sign	= 0;		// (0:plus)
//volatile uint15	gps_alt_ant_val		= 0;		// (int part)
//volatile uint1	gps_alt_geo_sign	= 0;		// (0:plus)
//volatile uint15	gps_alt_geo_val		= 0;		// (int part)
//volatile uint2	gps_valid			= 0;

// dummy
volatile uint1	dummy_tmr_out	= 0;

#define	UART_PULSE	(26042 / (1000 / CLK_FREQ_MHZ))		// 1/4 period of 9600bps=26041.666ns; unit clk
#define	RX_TIMEOUT	(1 * CLK_FREQ_HZ)					// unit: clk

////////////////////////////////////////////
// sub functions
////////////////////////////////////////////

void wait_tmr(uint32 tmr)	// unit clk
{
	uint32	t;
	ap_wait();
	for (t = 0; t < tmr; t++) {
		dummy_tmr_out	= 1- dummy_tmr_out;
	}
	ap_wait();
}


uint1 read_uart_rx(void)
{
	return uart_rx;
}


uint8 uart_receive_byte(void)
{
	uint4	i;
	uint8	data	= 0;
	uint8	dt;
	uint32	timer;

	do {
		ap_wait();
	} while (read_uart_rx() == 0);

	// wait start bit
	for (timer = 0; timer < RX_TIMEOUT; timer++) {
		ap_wait();
		if (read_uart_rx() == 0) {
			ap_wait();
			wait_tmr(UART_PULSE * 2);
			ap_wait();
			if (read_uart_rx() == 0) {
				break;	// continuous L
			}
		}
	}

	ap_wait();
	wait_tmr(UART_PULSE * 3);
	ap_wait();

	for (i = 0; i < 8; i++) {
		ap_wait();
		wait_tmr(UART_PULSE * 2);
		ap_wait();
		if (read_uart_rx() == 0)
			dt	= 0;
		else
			dt	= 0x80;

		ap_wait();
		data	= (data >> 1) | dt;
		ap_wait();
		wait_tmr(UART_PULSE * 2);
		ap_wait();
	}

	return (data);
}


////////////////////////////////////////////
// main
////////////////////////////////////////////
void gps_uart(void)
{
#pragma HLS TOP
#pragma HLS INTERFACE ap_none port=dummy_tmr_out		register

#pragma HLS INTERFACE ap_none port=uart_rx

#pragma HLS INTERFACE ap_none port=gps_time_hour		register
#pragma HLS INTERFACE ap_none port=gps_time_min			register
#pragma HLS INTERFACE ap_none port=gps_time_sec			register
//#pragma HLS INTERFACE ap_none port=gps_time_msec		register
//#pragma HLS INTERFACE ap_none port=gps_latitude_deg		register
//#pragma HLS INTERFACE ap_none port=gps_latitude_min 	register
//#pragma HLS INTERFACE ap_none port=gps_latitude_ns 		register
//#pragma HLS INTERFACE ap_none port=gps_longitude_deg	register
//#pragma HLS INTERFACE ap_none port=gps_longitude_min 	register
//#pragma HLS INTERFACE ap_none port=gps_longitude_ew 	register
//#pragma HLS INTERFACE ap_none port=gps_alt_ant_sign		register
//#pragma HLS INTERFACE ap_none port=gps_alt_ant_val		register
//#pragma HLS INTERFACE ap_none port=gps_alt_geo_sign		register
//#pragma HLS INTERFACE ap_none port=gps_alt_geo_val		register
//#pragma HLS INTERFACE ap_none port=gps_valid		 	register

	volatile uint1	val_negative;
	volatile uint32	val_int, val_frac;
	volatile uint8	ch;

	uint5	buf_time_hour;
	uint6	buf_time_min;
	uint6	buf_time_sec;
//	uint10	buf_time_msec;
//	uint8	buf_latitude_deg;
//	uint20	buf_latitude_min;
//	uint1	buf_latitude_ns;
//	uint8	buf_longitude_deg;
//	uint20	buf_longitude_min;
//	uint1	buf_longitude_ew;
//	uint1	buf_alt_ant_sign;
//	uint15	buf_alt_ant_val;
//	uint1	buf_alt_geo_sign;
//	uint15	buf_alt_geo_val;
//	uint2	buf_valid;

	wait_tmr(100 * CLK_FREQ_KHZ);	// wait GPS power up

	ap_wait();
	gps_time_hour		= 0;
	gps_time_min		= 0;
	gps_time_sec		= 0;
//	gps_time_msec		= 0;
//	gps_latitude_deg	= 0;
//	gps_latitude_min	= 0;
//	gps_latitude_ns		= 0;
//	gps_longitude_deg	= 0;
//	gps_longitude_min	= 0;
//	gps_longitude_ew	= 0;
//	gps_valid			= 0;
	ap_wait();

	// main loop
	while (1) {
		//		  time		lat			long		  vld,sat
		// $GPGGA,140436.000,5131.0970,N,00016.9597,W,1,06,1.2,38.4,M,47.0,M,,0000*7E
		// $GPGGA,131841.000,3533.0020,N,13940.4613,E,1,4,1.75,54.6,M,39.5,M,,*66
		if ((ch	= uart_receive_byte()) != '$')
			continue;
		if ((ch	= uart_receive_byte()) != 'G')
			continue;
		if ((ch	= uart_receive_byte()) != 'P')
			continue;
		if ((ch	= uart_receive_byte()) != 'G')
			continue;
		if ((ch	= uart_receive_byte()) != 'G')
			continue;
		if ((ch	= uart_receive_byte()) != 'A')
			continue;
		if ((ch	= uart_receive_byte()) != ',')
			continue;

		// $GPGGA,131841.000,3533.0020,N,13940.4613,E,1,4,1.75,54.6,M,39.5,M,,*66
		//       ^
		// time
		val_int	= 0;
		for	(ch	= uart_receive_byte(); ch != '.' &&	ch != ',' && ch	!= 0x0A; ch	= uart_receive_byte()) {
			val_int	= val_int * 10	+ (ch - '0');
		}
		val_frac	= 0;
		if (ch == '.') {
			for	(ch	= uart_receive_byte(); ch != ',' &&	ch != 0x0A;	ch = uart_receive_byte()) {
				val_frac	= val_frac	* 10 + (ch - '0');
			}
		}

		buf_time_hour	= val_int / 10000;
		buf_time_min	= (val_int % 10000) / 100;
		buf_time_sec	= val_int % 100;
//		buf_time_msec	= val_frac;

//		// $GPGGA,131841.000,3533.0020,N,13940.4613,E,1,4,1.75,54.6,M,39.5,M,,*66
//		//                  ^
//		// latitude
//		val_int	= 0;
//		if (ch == ',') {
//			for	(ch	= uart_receive_byte(); ch != '.' &&	ch != ',' && ch	!= 0x0A; ch	= uart_receive_byte()) {
//				val_int	= val_int *	10 + (ch - '0');
//			}
//		}
//		val_frac	= 0;
//		if (ch == '.') {
//			for	(ch	= uart_receive_byte(); ch != ',' &&	ch != 0x0A;	ch = uart_receive_byte()) {
//				val_frac	= val_frac * 10	+ (ch -	'0');
//			}
//		}
//		buf_latitude_ns	= 0;
//		if (ch == ',') {
//			ch	= uart_receive_byte();
//			if (ch != ',') {
//				if (ch == 'N') {
//					buf_latitude_ns	= 1;
//				}
//				ch	= uart_receive_byte();
//			}
//		}
//
//		buf_latitude_deg	= val_int / 100;
//		buf_latitude_min	= (uint20)(((val_int % 100)	* 10000) + val_frac) / 60;
//
//		// $GPGGA,131841.000,3533.0020,N,13940.4613,E,1,4,1.75,54.6,M,39.5,M,,*66
//		//                              ^
//		// longitude
//		val_int	= 0;
//		if (ch == ',') {
//			for	(ch	= uart_receive_byte(); ch != '.' &&	ch != ',' && ch	!= 0x0A; ch	= uart_receive_byte()) {
//				val_int	= val_int * 10	+ (ch -	'0');
//			}
//		}
//		val_frac	= 0;
//		if (ch == '.') {
//			for	(ch	= uart_receive_byte(); ch != ',' &&	ch != 0x0A;	ch = uart_receive_byte()) {
//				val_frac	= val_frac	* 10 + (ch - '0');
//			}
//		}
//		buf_longitude_ew	= 0;
//		if (ch == ',') {
//			ch	= uart_receive_byte();
//			if (ch != ',') {
//				if (ch == 'E') {
//					buf_longitude_ew	= 1;
//				}
//				ch	= uart_receive_byte();
//			}
//		}
//
//		buf_longitude_deg	= val_int /	100;
//		buf_longitude_min	= (uint20)(((val_int % 100)	* 10000) + val_frac) / 60;
//
//		// $GPGGA,131841.000,3533.0020,N,13940.4613,E,1,4,1.75,54.6,M,39.5,M,,*66
//		//                                           ^
//		// valid
//		buf_valid	= 0;
//		if (ch == ',') {
//			ch	= uart_receive_byte();
//			buf_valid	= ch - '0';
//
//			ch	= uart_receive_byte();
//		}
//
//		// $GPGGA,131841.000,3533.0020,N,13940.4613,E,1,4,1.75,54.6,M,39.5,M,,*66
//		//                                             ^
//		// cut # of satellites
//		if (ch == ',') {
//			ch	= uart_receive_byte();
//			while (ch != ',' && ch != 0x0A) {
//				ch = uart_receive_byte();
//			}
//		}
//
//		// $GPGGA,131841.000,3533.0020,N,13940.4613,E,1,4,1.75,54.6,M,39.5,M,,*66
//		//                                               ^
//		// cut # of HDOP
//		if (ch == ',') {
//			ch	= uart_receive_byte();
//			while (ch != ',' && ch != 0x0A) {
//				ch = uart_receive_byte();
//			}
//		}
//
//		// $GPGGA,131841.000,3533.0020,N,13940.4613,E,1,4,1.75,54.6,M,39.5,M,,*66
//		//                                                    ^
//		// ant alt
//		val_int			= 0;
//		val_negative	= 0;	// plus
//		if (ch == ',') {
//			for	(ch	= uart_receive_byte(); ch != '.' &&	ch != ',' && ch	!= 0x0A; ch	= uart_receive_byte()) {
//				if (ch == '-') {
//					val_negative	= 1;	// minus
//				}
//				else {
//					val_int	= val_int * 10	+ (ch - '0');
//				}
//			}
//		}
//		val_frac	= 0;
//		if (ch == '.') {
//			for	(ch	= uart_receive_byte(); ch != ',' &&	ch != 0x0A;	ch = uart_receive_byte()) {
//				val_frac	= val_frac	* 10 + (ch - '0');
//			}
//		}
//
//		buf_alt_ant_sign	= val_negative;
//		buf_alt_ant_val		= val_int & 0x7FFF;
//
//		// $GPGGA,131841.000,3533.0020,N,13940.4613,E,1,4,1.75,54.6,M,39.5,M,,*66
//		//                                                         ^
//		// cut unit
//		if (ch == ',') {
//			ch	= uart_receive_byte();
//			while (ch != ',' && ch != 0x0A) {
//				ch = uart_receive_byte();
//			}
//		}
//
//		// $GPGGA,131841.000,3533.0020,N,13940.4613,E,1,4,1.75,54.6,M,39.5,M,,*66
//		//                                                           ^
//		// geoid alt
//		val_int			= 0;
//		val_negative	= 0;	// plus
//		if (ch == ',') {
//			for	(ch	= uart_receive_byte(); ch != '.' &&	ch != ',' && ch	!= 0x0A; ch	= uart_receive_byte()) {
//				if (ch == '-') {
//					val_negative	= 1;	// minus
//				}
//				else {
//					val_int	= val_int * 10	+ (ch - '0');
//				}
//			}
//		}
//		val_frac	= 0;
//		if (ch == '.') {
//			for	(ch	= uart_receive_byte(); ch != ',' &&	ch != 0x0A;	ch = uart_receive_byte()) {
//				val_frac	= val_frac	* 10 + (ch - '0');
//			}
//		}
//
//		buf_alt_geo_sign	= val_negative;
//		buf_alt_geo_val		= val_int & 0x7FFF;

		///////////////////////////////////////////
		// output
		///////////////////////////////////////////
		ap_wait();
		gps_time_hour		= buf_time_hour;
		gps_time_min		= buf_time_min;
		gps_time_sec		= buf_time_sec;
//		gps_time_msec		= buf_time_msec;
//		gps_latitude_deg	= buf_latitude_deg;
//		gps_latitude_min	= buf_latitude_min;
//		gps_latitude_ns		= buf_latitude_ns;
//		gps_longitude_deg	= buf_longitude_deg;
//		gps_longitude_min	= buf_longitude_min;
//		gps_longitude_ew	= buf_longitude_ew;
//		gps_alt_ant_sign	= buf_alt_ant_sign;
//		gps_alt_ant_val		= buf_alt_ant_val;
//		gps_alt_geo_sign	= buf_alt_geo_sign;
//		gps_alt_geo_val		= buf_alt_geo_val;
//		gps_valid			= buf_valid;
		ap_wait();
	}	// while
}
