module gpsclock_top(
		rs, e, data,								// LCD
		gps_uart_rx,								// GPS
		led_out, dipsw_in, button_in,				// misc
		rst_n, clk
	);

// LCD
output			rs;
output			e;
output [3:0]	data;

// GPS
input			gps_uart_rx;

// misc
output [3:0]	led_out;
input  [3:0]	dipsw_in;
input  [3:0]	button_in;

// clk/reset
input			rst_n;
input			clk;

wire [4:0]		gps_time_hour;
wire [5:0]		gps_time_min;
wire [5:0]		gps_time_sec;

///////////////////////////////////////////////////////////////////////
// clock and reset
///////////////////////////////////////////////////////////////////////
reg				clk_reg;
wire			sysclk;

always @(posedge clk) begin
	clk_reg	<= ~clk_reg;
end

assign	sysclk	= clk_reg;					// 50MHz

reg  [1:0]		rst_n_sync_reg;
wire			rst_n_sync;
wire			rst_p_sync;

always @(posedge sysclk) begin
	rst_n_sync_reg	<= {rst_n_sync_reg[0], rst_n};
end

assign	rst_n_sync	= rst_n_sync_reg[1];
assign	rst_p_sync	= ~rst_n_sync_reg[1];

///////////////////////////////////////////////////////////////////////
// lcd
///////////////////////////////////////////////////////////////////////
lcddrv_gps_lcddrv_gps u1 (
	.ap_clk(sysclk),
	.ap_rst(rst_p_sync),
	.ap_start(1'b1),

	.rs(rs),
	.en(e),
	.data(data),

	.dummy_tmr_out_i(1'b0),

	.gps_time_hour(gps_time_hour),
	.gps_time_min(gps_time_min),
	.gps_time_sec(gps_time_sec)
);

///////////////////////////////////////////////////////////////////////
// gps
///////////////////////////////////////////////////////////////////////
reg  [1:0]		gps_uart_rx_sync_reg;
wire			gps_uart_rx_sync;

always @(posedge sysclk) begin
	gps_uart_rx_sync_reg	<= {gps_uart_rx_sync_reg[0], gps_uart_rx};
end

assign	gps_uart_rx_sync	= gps_uart_rx_sync_reg[1];

gps_uart_gps_uart u2 (
	.ap_clk(sysclk),
	.ap_rst(rst_p_sync),
	.ap_start(1'b1),
	.dummy_tmr_out_i(1'b0),

	.uart_rx(gps_uart_rx_sync),

	.gps_time_hour(gps_time_hour),
	.gps_time_min(gps_time_min),
	.gps_time_sec(gps_time_sec)
);

///////////////////////////////////////////////////////////////////////

assign	led_out	= {~gps_uart_rx_sync, 1'b0, 1'b1, rst_n_sync};

endmodule
