#include <ap_cint.h>
#include <ap_utils.h>

#define	CLK_FREQ_MHZ	50
#define	CLK_FREQ_KHZ	50000

////////////////////////////////////////////
// port
////////////////////////////////////////////

// LCD
volatile uint1	rs		= 0;
volatile uint1	en		= 0;
volatile uint4	data	= 0;

// data buffer
volatile uint5	gps_time_hour;
volatile uint6	gps_time_min;
volatile uint6	gps_time_sec;

// dummy
volatile uint1	dummy_tmr_out	= 0;

////////////////////////////////////////////
// main
////////////////////////////////////////////
void wait_tmr(uint32 tmr)	// unit clk
{
	uint32	t;
	ap_wait();
	for (t = 0; t < tmr; t++) {
		dummy_tmr_out	= 1- dummy_tmr_out;
	}
	ap_wait();
}


void lcd_send_cmd(uint1 mode, uint4 wd)
{
	ap_wait();
	en		= 0;
	ap_wait();
	wait_tmr(10 * CLK_FREQ_MHZ);

	ap_wait();
	en		= 0;
	rs		= mode;
	data	= wd;
	ap_wait();
	wait_tmr(10 * CLK_FREQ_MHZ);

	ap_wait();
	en		= 1;
	rs		= mode;
	data	= wd;
	ap_wait();
	wait_tmr(10 * CLK_FREQ_MHZ);

	ap_wait();
	en		= 0;
	rs		= mode;
	data	= wd;
	ap_wait();
	wait_tmr(10 * CLK_FREQ_MHZ);
}


void init_lcd()
{
	wait_tmr(10 * CLK_FREQ_KHZ);
	lcd_send_cmd(0, 0x3);
	wait_tmr(5 * CLK_FREQ_KHZ);
	lcd_send_cmd(0, 0x3);
	wait_tmr(500 * CLK_FREQ_MHZ);
	lcd_send_cmd(0, 0x3);
	wait_tmr(500 * CLK_FREQ_MHZ);

	lcd_send_cmd(0, 0x2);		// 4bit i/f
	lcd_send_cmd(0, 0x0);
	lcd_send_cmd(0, 0x8);		// 2lines, 5x8 font
	wait_tmr(100 * CLK_FREQ_MHZ);

	lcd_send_cmd(0, 0x0);
	lcd_send_cmd(0, 0xC);		// disp on/cursor off/blink off
	wait_tmr(100 * CLK_FREQ_MHZ);

	lcd_send_cmd(0, 0x0);
	lcd_send_cmd(0, 0x6);
	wait_tmr(100 * CLK_FREQ_MHZ);

	lcd_send_cmd(0, 0x0);
	lcd_send_cmd(0, 0x1);		// disp clr
	wait_tmr(2 * CLK_FREQ_KHZ);

	lcd_send_cmd(0, 0x0);
	lcd_send_cmd(0, 0x2);		// increment/shift off
	wait_tmr(100 * CLK_FREQ_MHZ);
}


uint8 bin2char(uint4 val)
{
	uint8	retval;

	switch (val) {
	case 0:		retval	= '0'; break;
	case 1:		retval	= '1'; break;
	case 2:		retval	= '2'; break;
	case 3:		retval	= '3'; break;
	case 4:		retval	= '4'; break;
	case 5:		retval	= '5'; break;
	case 6:		retval	= '6'; break;
	case 7:		retval	= '7'; break;
	case 8:		retval	= '8'; break;
	case 9:		retval	= '9'; break;
	case 10:	retval	= 'A'; break;
	case 11:	retval	= 'B'; break;
	case 12:	retval	= 'C'; break;
	case 13:	retval	= 'D'; break;
	case 14:	retval	= 'E'; break;
	default:	retval	= 'F';
	}

	return retval;
}


void lcddrv_gps()
{
#pragma HLS TOP
#pragma HLS INTERFACE ap_none port=dummy_tmr_out	register

#pragma HLS INTERFACE ap_none port=rs	register
#pragma HLS INTERFACE ap_none port=en	register
#pragma HLS INTERFACE ap_none port=data	register

#pragma HLS INTERFACE ap_none port=gps_time_hour
#pragma HLS INTERFACE ap_none port=gps_time_min
#pragma HLS INTERFACE ap_none port=gps_time_sec

	uint8	pos;
	uint8	dt;
	uint8	hour, min, sec;

	init_lcd();

	while (1) {
		hour	= gps_time_hour;
		min		= gps_time_min;
		sec		= gps_time_sec;
		ap_wait();
		hour	+= 9;		// JST
		if (hour > 23)
			hour	-= 24;

		// line 1
		lcd_send_cmd(0, 0x8);
		lcd_send_cmd(0, 0x0);		// set address

		// hour
		dt	= hour / 10;
		dt	= bin2char((uint4)dt);
		lcd_send_cmd(1, (dt >> 4) & 0xF);
		lcd_send_cmd(1, dt & 0xF);

		dt	= hour % 10;
		dt	= bin2char((uint4)dt);
		lcd_send_cmd(1, (dt >> 4) & 0xF);
		lcd_send_cmd(1, dt & 0xF);

		dt	= ':';
		lcd_send_cmd(1, (dt >> 4) & 0xF);
		lcd_send_cmd(1, dt & 0xF);

		// min
		dt	= min / 10;
		dt	= bin2char((uint4)dt);
		lcd_send_cmd(1, (dt >> 4) & 0xF);
		lcd_send_cmd(1, dt & 0xF);

		dt	= min % 10;
		dt	= bin2char((uint4)dt);
		lcd_send_cmd(1, (dt >> 4) & 0xF);
		lcd_send_cmd(1, dt & 0xF);

		dt	= ':';
		lcd_send_cmd(1, (dt >> 4) & 0xF);
		lcd_send_cmd(1, dt & 0xF);

		// sec
		dt	= sec / 10;
		dt	= bin2char((uint4)dt);
		lcd_send_cmd(1, (dt >> 4) & 0xF);
		lcd_send_cmd(1, dt & 0xF);

		dt	= sec % 10;
		dt	= bin2char((uint4)dt);
		lcd_send_cmd(1, (dt >> 4) & 0xF);
		lcd_send_cmd(1, dt & 0xF);

		ap_wait();
		wait_tmr(5000000);			// 100ms@50MHz
	}

}

// end of file
